// SimpleHydro.h: interface for the CSimpleHydro class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SIMPLEHYDRO_H__3E14506F_FA2C_11D7_9CF8_005056063E62__INCLUDED_)
#define AFX_SIMPLEHYDRO_H__3E14506F_FA2C_11D7_9CF8_005056063E62__INCLUDED_


#include "CHydro.h"
#include "math.h"
#include "constants.h"

#include "tnt.h"
#include <map>

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


/** @brief Simple Hydrodynamic module

	@author M.Gainville - TINA Project - Institut Franais du Ptrole - 2005 */
// ======================================================================================
class CSimpleHydro : public CHydro  
{
public:
	/** Constructors & Destructor */
	//@{
	/** Destructor */
	virtual ~CSimpleHydro();
	/** @brief Constructor of the Virtual Class.
	@param frictionFactor : friction factor.*/
	CSimpleHydro();
	//@}

	/// <summary>Method to define which hydro is in use</summary>
	/// <returns>SIMP</returns>
	/// <exception>No exception</exception>
	enum hydroCst::modelHydro get_hydroID() throw();

	/// <summary>initialize hydro model given a data model</summary>
	/// <param name="phydroData">data model</param>
	/// <returns>void</returns>
	/// <remarks>Nothing to do</remarks>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	//void initialize(CHydroData* phydroData) throw(...){}

	/** Methods */
	//@{
	/** @brief Performe hydrodynamic calculations knowing the sperficial velocities.
	@param phaseQualifiers The phase ID of the present phases, dimension equal to nbrOfPresentPhases.
	@return . */
	void hydroUk(double diameter, double roughness, double slopeSinus, int nbrOfPresentPhases,
						const TNT::Array1D<EPhases> &phaseQualifiers,const TNT::Array1D<double> &superficialVelocities,
						const TNT::Array1D<double> &densities,const TNT::Array1D<double> &viscosities,
						const TNT::Array2D<double> &surfaceTensions,const TNT::Array1D<double> &heatCapacities,
						const TNT::Array1D<double> &conductivities) throw(...);
	/** @brief Performe hydrodynamic calculations knowing the phase volumetric fractions and the barycentric velocities.
	@param phaseQualifiers The phase ID of the present phases, dimension equal to nbrOfPresentPhases.
	@return . */
	HRESULT hydroRkVm(const double diameter,const double roughness,const double slopeSinus,const int nbrOfPresentPhases,
						const TNT::Array1D<EPhases> &phaseQualifiers,const TNT::Array1D<double> &localVolumetricFractions,
						double barycentricVelocity,
						const TNT::Array1D<double> &densities,const TNT::Array1D<double> &viscosities,
						const TNT::Array2D<double> &surfaceTensions,const TNT::Array1D<double> &heatCapacities,
						const TNT::Array1D<double> &conductivities);
	/** @brief Perform hydrodynamic calculations knowing the phase volumetric fractions & the mixture Velocity.
		@param diameter   Pipe diameter
		@param roughness  Pipe roughness
		@param slopeSinus Pipe slope sinus
		@param nbrOfPresentPhases
		@param 
		@return */
	HRESULT hydroRkUm(const double diameter, const double roughness, const double slopeSinus,
					  const int nbrOfPresentPhases, const TNT::Array1D<EPhases> &phaseQualifiers,
					  const TNT::Array1D<double> &volumetricFractions, const double          mixtureVelocity,
					  const TNT::Array1D<double> &densities,           const TNT::Array1D<double> &viscosities,
					  const TNT::Array2D<double> &surfaceTensions,     const TNT::Array1D<double> &heatCapacities,
					  const TNT::Array1D<double> &conductivities);

	/** @brief Compute hydrodynamic properties (localPressureDrop, internalFluidResistitvity ...) .
	@return . */
	HRESULT computeProperties() throw(...);	
	/** @brief Compute hydrodynamic devivatives.
	@return . */
	HRESULT computeDerivatives();

	/** @brief Compute hydrodynamic derivatives in base RkVm */
	void computeDerivativesRkVm();

	/** @bried Compute hydrodynamic derivatives in base RkUm */
	void computeDerivativesRkUm();
	//@}

private :

	/** Methods */
	//@{
	/** @brief Compute derivatives .*/
	void computeSimpleHydroDerivatives();
	/** @brief Check present phases and update m_isPresentPhases .*/
	HRESULT checkPresentPhases();
	/** @brief Compute phase nusselt number .
	@param phase reynolds number.
	@param phase prandtl number.
	@return nusselt number. */
	double nusselt(double phaseReynolds,double phasePrandtl) const;
	/** @brief Compute phase nusselt number and derivatives.
	@param phase reynolds number.
	@param phase prandtl number.
	@param derivatives.
	@return nusselt number. */
	double derNusselt(double phaseReynolds,double  phasePrandtl,double  *derNuDerPhaseReynold,double  *derNuDerPhasePrandtl) const;
	double absVal(double x) const;
	double sgnVal(double x) const;
	//@}

	std::map<unsigned int, std::pair<EHydroFlowPatterns, double> > m_mapHydroProp;

    double m_frictionFactor;
	TNT::Array1D<bool> m_isPresentPhases;
	
};

#endif // !defined(AFX_SIMPLEHYDRO_H__3E14506F_FA2C_11D7_9CF8_005056063E62__INCLUDED_)
