// Exception.h: interface for the IException class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EXCEPTION_H__5F2A897B_F242_11D7_9CF2_005056063E62__INCLUDED_)
#define AFX_EXCEPTION_H__5F2A897B_F242_11D7_9CF2_005056063E62__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <comdef.h>
#include <string>
#include "CCapeUser.h"

#if defined __TESTCMYEXCEPTION_H__
class TestCMyException;
#endif
#if defined __CAPE_TEST_EXCEPTIONS__
class CCOExceptions;
#endif


namespace R2PException{
	/// <summary>Exception handling class: abstract class to CCapeException and CForwardException</summary> 
	class CMyException  
	{
#if defined (__TESTCMYEXCEPTION_H__)
		friend TestCMyException;
#endif
#if defined (__CAPE_TEST_EXCEPTIONS__)
		friend CCOExceptions;
#endif
	public:
		/// <summary>Constructor</summary>
		CMyException();

		/// <summary>Copy Constructor</summary>
		/// <param name="_ex">object to copy</param>
		CMyException(const CMyException& _ex);

		/// <summary>Constructor</summary>
		/// <param name="error_code">error code that symbolizes the CAPE-OPEN error</param>
		/// <param name="name">a brief description of the error</param>
		/// <param name="desc">description of the error</param>
		CMyException(HRESULT error_code, const std::wstring& name, const wchar_t* desc, ...);

		/// <summary>Constructor</summary>
		/// <param name="error_code">CAPE-OPEN error code</param>
		/// <param name="pdis">pointer to the interface component that has thrown error</param>
		CMyException(HRESULT error_code, const IDispatchPtr& pdis);

		/// <summary>Destructor</summary>
		virtual ~CMyException();

		/// <summary>Get brief description</summary>
		/// <param name="desc">brief description</param>
		/// <returns>void</returns>
		/// <remarks>from ECapeRoot</remarks>
		HRESULT get_name(std::wstring& name);

		/// <summary>Set brief description</summary>
		/// <param name="desc">brief description</param>
		/// <returns>void</returns>
		/// <remarks>from ECapeRoot</remarks>
		HRESULT set_name(const std::wstring& name);

		/// <summary>Get legacy error code</summary>
		/// <param name="code">legacy error code</param>
		/// <returns>void</returns>
		/// <remarks>from ECapeUser</remarks>
		HRESULT get_legacyCode(long& code);

		/// <summary>Get CO error code</summary>
		/// <param name="code">CO error code</param>
		/// <returns>void</returns>
		HRESULT get_COCode(long& code);

		/// <summary>Get description</summary>
		/// <param name="description">description</param>
		/// <returns>void</returns>
		/// <remarks>from ECapeUser</remarks>
		HRESULT get_description(std::wstring& description);

		/// <summary>Set description</summary>
		/// <param name="description">description</param>
		/// <returns>void</returns>
		/// <remarks>from ECapeUser</remarks>
		HRESULT set_description(const std::wstring& description);

		/// <summary>Add description string</summary>
		/// <param name="description">description following printf format</param>
		/// <param name="...">va_list that describes dynamic parameter from description argument</param>
		/// <returns>void</returns>
		HRESULT add_description(const wchar_t* description, ...);

		/// <summary>Get scope</summary>
		/// <param name="scope">scope</param>
		/// <returns>void</returns>
		/// <remarks>from ECapeUser</remarks>
		HRESULT get_scope(std::wstring& scope);

		/// <summary>Set scope</summary>
		/// <param name="scope">scope</param>
		/// <returns>void</returns>
		/// <remarks>from ECapeUser</remarks>
		HRESULT set_scope(const std::wstring& scope);

		/// <summary>Get the name of the interface where the error is thrown</summary>
		/// <param name="interfaceName">name of the interface</param>
		/// <returns>void</returns>
		/// <remarks>from ECapeUser</remarks>
		HRESULT get_interfaceName(std::wstring& interfaceName);

		/// <summary>Set the name of the interface where the error is thrown</summary>
		/// <param name="interfaceName">name of the interface</param>
		/// <returns>void</returns>
		/// <remarks>from ECapeUser</remarks>
		HRESULT set_interfaceName(const std::wstring& interfaceName);

		/// <summary>Get the name of the operation where the error is thrown</summary>
		/// <param name="operation">name of the operation</param>
		/// <returns>void</returns>
		/// <remarks>from ECapeUser</remarks>
		HRESULT get_operation(std::wstring& operation);

		/// <summary>Set the name of the operation where the error is thrown</summary>
		/// <param name="operation">name of the operation</param>
		/// <returns>void</returns>
		/// <remarks>from ECapeUser</remarks>
		HRESULT set_operation(const std::wstring& operation);

		/// <summary>Get a string where more information on the error can be found</summary>
		/// <param name="moreInfo">string of the information</param>
		/// <returns>void</returns>
		/// <remarks>from ECapeUser</remarks>
		HRESULT get_moreInfo(std::wstring& moreInfo);

		/// <summary>Set a string where more information on the error can be found</summary>
		/// <param name="moreInfo">string of the information</param>
		/// <returns>void</returns>
		/// <remarks>from ECapeUser</remarks>
		HRESULT set_moreInfo(const std::wstring& moreInfo);

		/// <summary>get error specific class attribute</summary>
		/// <param name="ParamName">Name of parameter that throw error</param>
		/// <returns>void</returns>
		/// <remarks>from ECapeBadCOParameter</remarks>
		HRESULT get_parameterName(std::wstring& ParamName);

		/// <summary>Set error specific class attribute</summary>
		/// <param name="ParamName">Name of parameter that throw error</param>
		/// <returns>void</returns>
		/// <remarks>from ECapeBadCOParameter</remarks>
		HRESULT set_parameterName(const std::wstring& ParamName);

		/// <summary>get error specific class attribute</summary>
		/// <param name="theParam">pointer to Parameter interface</param>
		/// <returns>void</returns>
		/// <remarks>This method calls AddRef() on the ICapeParameter interface. Be sure that client will release it.
		/// <para>from ECapeBadCOParameter</para>
		/// </remarks>
		HRESULT get_parameter(IDispatch** theParam);

		/// <summary>set error specific class attribute</summary>
		/// <param name="theParam">pointer to Parameter interface</param>
		/// <returns>void</returns>
		/// <remarks>This method calls AddRef() on the ICapeParameter interface. Be sure that client will release it.
		/// <para>from ECapeBadCOParameter</para>
		/// </remarks>
		HRESULT set_parameter(const IDispatch* theParam);

		/// <summary>get the position from collection where argument is bad</summary>
		/// <param name="pos">the position</param>
		/// <returns>void</returns>
		/// <remarks>from ECapeBadArgument</remarks>
		HRESULT get_position(short& pos);

		/// <summary>set the position from collection where argument is bad</summary>
		/// <param name="pos">the position</param>
		/// <returns>void</returns>
		/// <remarks>from ECapeBadArgument</remarks>
		HRESULT set_position(const short& pos);

		/// <summary>get error specific class attribute</summary>
		/// <param name="lbound">lower bound value to set</param>
		/// <returns>void</returns>
		/// <remarks>from ECapeBoundaries</remarks>
		HRESULT get_LowerBound(double& lbound);

		/// <summary>get error specific class attribute</summary>
		/// <param name="ubound">upper bound value to set</param>
		/// <returns>void</returns>
		/// <remarks>from ECapeBoundaries</remarks>
		HRESULT get_UpperBound(double& ubound);

		/// <summary>get error specific class attribute</summary>
		/// <param name="value">current value to set</param>
		/// <returns>void</returns>
		/// <remarks>from ECapeBoundaries</remarks>
		HRESULT get_value(double& value);

		/// <summary>get error specific class attribute</summary>
		/// <param name="type">type to set</param>
		/// <returns>void</returns>
		/// <remarks>from ECapeBoundaries</remarks>
		HRESULT get_Type(std::wstring& type);

		/// <summary>set error specific class attribute</summary>
		/// <param name="lbound">lower bound value to set</param>
		/// <param name="ubound">upper bound value to set</param>
		/// <param name="value">current value to set</param>
		/// <param name="type">type to set</param>
		/// <returns>void</returns>
		/// <remarks>from ECapeBoundaries</remarks>
		HRESULT set_Bound(const double& lbound, const double& ubound, const double& value, const std::wstring& type);

		/// <summary>get error specific class attribute</summary>
		/// <param name="ReqOp">Name of the prerequisite operation</param>
		/// <returns>void</returns>
		/// <remarks>from ECapeBadInvOrder</remarks>
		HRESULT get_requestedOperation(std::wstring& ReqOp);

		/// <summary>set error specific class attribute</summary>
		/// <param name="ReqOp">Name of the prerequisite operation</param>
		/// <returns>void</returns>
		/// <remarks>from ECapeBadInvOrder</remarks>
		HRESULT set_requestedOperation(const std::wstring& ReqOp);

		/// <summary>get error specific class attribute</summary>
		/// <param name="iName">name of item</param>
		/// <returns>void</returns>
		/// <remarks>from ECapePersistenceNotFound</remarks>
		HRESULT get_itemName(std::wstring& iName);

		/// <summary>set error specific class attribute</summary>
		/// <param name="iName">name of item</param>
		/// <returns>void</returns>
		/// <remarks>from ECapePersistenceNotFound</remarks>
		HRESULT set_itemName(const std::wstring& iName);
		

	protected:
		/// <summary>exception can be CAPE-OPEN or a call to CO component</summary>
		/// <remarks>this struct manage a garbage collector in order to avoid unnecessary copy</remarks>
		struct srefUnion{
			CCapeUser* COEx;
			unsigned int _ref;
			HRESULT _COCode;
		}* _refError;
	};
}

#endif // !defined(AFX_EXCEPTION_H__5F2A897B_F242_11D7_9CF2_005056063E62__INCLUDED_)
